clear all
close all 
tic
p              = LoadPath('CA')                                            ;

%% Loading Experimental data 
%%% load experimental data 
%%% n is the reference of the data in the file 
n             = 'kd_IrHH_Ha_temp_267_[2]'                                  ; %                                                                               
File          = "IrHH_data\kd_IrHH_temp_[2]\"                              ; %  
data_in       = load(File + n + '.txt','-ASCII')                           ;
data.dmix0    = data_in(:,1)                                               ;
data.dmix     = data.dmix0-data.dmix0(1)                                   ; % Here we take in consideration that first point starts from 0 and all dmix are adjusted
data.Ha       = data_in(:,2)                                               ; % Integrals of [2]-Ha   
data.Hb       = data_in(:,3)                                               ; % Integrals of [2]-Hb  
data.H2       = data_in(:,4)                                               ; % Integrals of free 
data.Ha2      = data_in(:,5)                                               ; % Integrals of [1]-Ha   
data.Hb2      = data_in(:,6)                                               ; % Integrals of [1]-Hb  
%data.H3       = data_in(:,5)                                              ;
a = size(data.dmix0);
data.H3       = rand(a(1),1)*0.005                                         ; % Integrals of H3 


% Choosing the fitting Model 

Model_types{1} = 'IrHH->H2'                                                ;
Model_types{2} = 'IrHH->H2 with 3R'                                        ;
Model_types{3} = 'IrHH->H2 with 3R-alpha'                                  ;
Model_types{4} = 'IrHH->H2_normal'                                         ;
Model_types{5} = 'IrHH->H2 M 6*6'                                          ;
Model_types{6} = 'IrHH->H2 M 5*5'                                          ;
Model_types{7} = 'IrHH->H2 M 4*4'                                          ;
Model_types{8} = '([1]<->[2])->H2 pre 3*3'                                 ;

Model_type     = Model_types  {6}                                          ; % choose the Fitting Model

            
switch Model_type
         
       case 'IrHH->H2'                                                     % for random Rf and Re 
            
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            H2 = 40 ;   Ir0  = 4;     MHa = 1;                MHb =  data.Hb(1)  ;   MH2 = 0;%data.H2(1)                      ; % data.Ha(1)  ;          data.Hb(1)  ;   data.H2(1) 
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1],3*Nt,1)                                                        ;
            ydata                         = [data.Ha; data.Hb; data.H2 ]                                                             ; % the three sets of ydata values
            i_params                      = [ 0.03, 0.3, 0.02, 0.3]                                                                        ; %   kd, kex, RH2, kaprime , Rb
            
            % Initial guess                                                                    (MHa, MHb, MH2,          kd,        kex,          RH2,           kaprime ,          RHab, x)
            fitfunc                       = @(i_params, x)SABRE_IrHH_threesite_exchange_Lmatrix(MHa, MHb, MH2, i_params(1), i_params(2), i_params(3), i_params(1)*Ir0/H2, i_params(4), x)  ;
            y0                            = SABRE_IrHH_threesite_exchange_Lmatrix(MHa, MHb, MH2, i_params(1), i_params(2),i_params(3),i_params(1)*Ir0/H2, i_params(4),xdata_fit )          ;
            semilogx(data.dmix, data.Ha, 'o', data.dmix, data.Hb, 'o',data.dmix, data.H2, 'o',data.dmix,reshape(y0,Nt,3))            ;                                                                                                             
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, i_params)                                               ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("results: \n")                                                                                                   ;
            fprintf("Model IrHH->H2 has kd = %.3f ± %.3f s-1,kex = %.3f ± %.3f s-1, kaprime = %.3f ± %.3f s-1 , RH2 = %.3f ± %.3f s-1, RHab = %.3f ± %.3f s-1 \n",beta1(1),CSE(1),beta1(2),CSE(2),beta1(1)*Ir0/H2,CSE(1)*Ir0/H2,beta1(3),CSE(3),beta1(4),CSE(4))

     case 'IrHH->H2 with 3R'                                                     % for random Rf and Re 
            
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            H2 = 100 ;   Ir0  = 3;     MHa = 1;                MHb =  data.Hb(1)  ;   MH2 = 0 ;%data.H2(1)                      ; % data.Ha(1)  ;          data.Hb(1)  ;   data.H2(1) 
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1],3*Nt,1)                                                        ;
            ydata                         = [data.Ha; data.Hb; data.H2 ]                                                             ; % the three sets of ydata values
            i_params                      = [ 0.032, 0.651,0.054, 0.368, 0.009]                                                                        ; %   kd, kex, RH2, kaprime , Rb
            
            % Initial guess                                                                    (MHa, MHb, MH2,          kd,        kex,          RH2,           kaprime ,          RHa,       RHb, x)
            fitfunc                       = @(i_params, x)SABRE_IrHH_threesite_exchange_Lmatrix_3R(MHa, MHb, MH2, i_params(1), i_params(2), i_params(3), i_params(1)*Ir0/H2, i_params(4),i_params(5), x)  ;
            y0                            = SABRE_IrHH_threesite_exchange_Lmatrix_3R(MHa, MHb, MH2, i_params(1), i_params(2),i_params(3),i_params(1)*Ir0/H2, i_params(4),i_params(5),xdata_fit )          ;
            semilogx(data.dmix, data.Ha, 'o', data.dmix, data.Hb, 'o',data.dmix, data.H2, 'o',data.dmix,reshape(y0,Nt,3))            ;                                                                                                             
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, i_params)                                               ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("results: \n")                                                                                                   ;
            fprintf("Model IrHH->H2 with 3R has kd = %.3f ± %.3f s-1,kex = %.3f ± %.3f s-1, kaprime = %.3f ± %.3f s-1 , RH2 = %.3f ± %.3f s-1, RHa = %.3f ± %.3f s-1, RHb = %.3f ± %.3f s-1 \n",beta1(1),CSE(1),beta1(2),CSE(2),beta1(1)*Ir0/H2,CSE(1)*Ir0/H2,beta1(3),CSE(3),beta1(4),CSE(4),beta1(5),CSE(5))   
    
    case 'IrHH->H2 with 3R-alpha'                                                     % for random Rf and Re 
            
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            H2 = 100 ;   Ir0  = 3;     MHa = 1;                MHb =  data.Hb(1)  ;   MH2 = 0 ; alpha = 2;%data.H2(1)                      ; % data.Ha(1)  ;          data.Hb(1)  ;   data.H2(1) 
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1],3*Nt,1)                                                        ;
            ydata                         = [data.Ha; data.Hb; data.H2 ]                                                             ; % the three sets of ydata values
            i_params                      = [ 6, 0.651,0.054, 0.368, 0.9,0.1]                                                  ; %   kd, kex, RH2, kaprime , Rb
            
            % Initial guess                                                                    (MHa, MHb, MH2,          kd,        kex,          RH2,           kaprime ,          RHa,       RHb, x)
            fitfunc                       = @(i_params, x)SABRE_IrHH_threesite_exchange_Lmatrix_3R_alpha(MHa, MHb, MH2, i_params(1), i_params(2), i_params(3), i_params(4), i_params(5),i_params(6), x, alpha)  ;
            y0                            = SABRE_IrHH_threesite_exchange_Lmatrix_3R_alpha(MHa, MHb, MH2, i_params(1), i_params(2),i_params(3),i_params(4),i_params(5),i_params(6),xdata_fit,alpha)             ;
            semilogx(data.dmix, data.Ha, 'o', data.dmix, data.Hb, 'o',data.dmix, data.H2, 'o',data.dmix,reshape(y0,Nt,3))            ;                                                                                                             
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, i_params)                                               ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("results: \n")                                                                                                   ;
            fprintf("Model IrHH->H2 with 3R has kd = %.3f ± %.3f s-1,kex = %.3f ± %.3f s-1, kaprime = %.3f ± %.3f s-1 , RH2 = %.3f ± %.3f s-1, RHa = %.3f ± %.3f s-1, RHb = %.3f ± %.3f s-1 \n",beta1(1),CSE(1),beta1(2),CSE(2),beta1(1)*Ir0/H2,CSE(1)*Ir0/H2,beta1(3),CSE(3),beta1(4),CSE(4),beta1(5),CSE(5))   
            
    case 'IrHH->H2_normal'
                        
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            MHa = data.Ha(1) ;        MHb = data.Hb(1);   MH2 = data.H2(1) ;  % 
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1],3*Nt,1)                                                        ;
            ydata                         = [data.Ha; data.Hb; data.H2 ]                                                             ; % the three sets of ydata values
            i_params                      = [ 0.04, 2.3, 1.29, 0.05, 1.53]                                                           ; %   kd, kex, RH2, kaprime , Rb
            
            % Initial guess                                                                    (MHa, MHb, MH2,          kd,        kex,          RH2,           kaprime ,          RHab, x)
            fitfunc                       = @(i_params, x)SABRE_IrHH_threesite_exchange_Lmatrix(MHa, MHb, MH2, i_params(1), i_params(2), i_params(3), i_params(4), i_params(5), x)  ;
            y0                            = SABRE_IrHH_threesite_exchange_Lmatrix(MHa, MHb, MH2, i_params(1), i_params(2),i_params(3),i_params(4), i_params(5),xdata_fit )          ;
            semilogx(data.dmix, data.Ha, 'o', data.dmix, data.Hb, 'o',data.dmix, data.H2, 'o',data.dmix,reshape(y0,Nt,3))            ;                                                                                                             
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, i_params)                                               ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("results: \n")                                                                                                   ;
            fprintf("Model IrHH->H2_normal has kd = %.3f ± %.3f s-1,kex = %.3f ± %.3f s-1, RH2 = %.3f ± %.3f s-1 , kaprime = %.3f ± %.3f s-1, RHab = %.3f ± %.3f s-1 \n",beta1(1),CSE(1),beta1(2),CSE(2),beta1(3),CSE(3),beta1(4),CSE(4),beta1(5),CSE(5))
        
     case 'IrHH->H2 M 6*6'
                        
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            MH1a = data.Ha(1);       MH1b = data.Hb(1);   MH2 = data.H2(1);     MH2a = data.Ha2(1);        MH2b = data.Hb2(1);       MH3ab = data.H3(1)   ; %
            k12  = 0.284;  k21 = 0.565 ; %
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1,1,1,1],6*Nt,1)                                                  ;
            ydata                         = [data.Ha; data.Hb; data.Ha2; data.Hb2; data.H3; data.H2]                                 ; % the three sets of ydata values
            I                             = [2, 20, 1, 1, 0.5,1, -0.284, 0.565, 0.8, 0.8, 5, 1.3]                                ; % Initial parameters  (kd1,kd2,kd3,kaprime1,kaprime2,kaprime3, kex1, kex2,R1,R2,R3,Rf,x)
            %I                             = [0.6, 0.5, 0, 0.2, 0.5]                                ; % Initial parameters  (kd1,kd2,kd3,kaprime1,kaprime2,kaprime3, kex1, kex2,R1,R2,R3,Rf,x) 
            % Initial guess                                                                (MH1a, MH1b, MH2a, MH2b, MH3ab, MH2,k12,k21,kd1,kd2,kd3, kaprime1,kaprime2,kaprime3, kex1, kex2, R1,R2,R3,Rf,x)   
            fitfunc                       = @(I, x)SABRE_hydrogen_exchange_Lmatrix_6(MH1a, MH1b, MH2a, MH2b, MH3ab, MH2,k12, k21, I(1), I(2), I(3), I(4), I(5), I(6), I(7), I(8),I(9), I(10), I(11), I(12), x)  ;
             %                                                                             (MH1a, MH1b, MH2a, MH2b, MH3ab, MH2,k12, k21,  kd1,  kd2,  kd3, kaprime1,kaprime2,kaprime3, kex1, kex2,    R1,    R2,   R3,  Rf, x) 
           % fitfunc                       = @(I, x)SABRE_hydrogen_exchange_Lmatrix_6(MH1a, MH1b, MH2a, MH2b, MH3ab, MH2,k12, k21, 0, -0.2,    1,   0.7, 0.565,    4, 1.2,I(1), I(2), I(3),     I(4),    I(5), x)  ;
            y0                            = SABRE_hydrogen_exchange_Lmatrix_6(MH1a, MH1b, MH2a, MH2b, MH3ab, MH2,              k12, k21, 0, -0.2,  0.1,   0.7, 0.565,    4, 1.2,I(1), I(2), I(3),     I(4),    I(5), xdata_fit )       ;
            semilogx(data.dmix, data.Ha, 'oblack', data.dmix, data.Hb, 'or',data.dmix, data.Ha2, 'og',data.dmix, data.Hb2,'Xg',data.dmix, data.H3,'Xr',data.dmix, data.H2,'Ob')            ; %,data.dmix,reshape(y0,Nt,6)
            hold on
            semilogx(data.dmix, y0(1:29), '--black', data.dmix, y0(30:58), '--r',data.dmix, y0(59:87), '--g',data.dmix, y0(88:116),'green',data.dmix, y0(117:145),'red',data.dmix, y0(146:174),'blue')            ;
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, I)                                                      ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("Results of Model IrHH->H2 M 6*6 has: \n")                                                                                                           ;
            fprintf("kd1 = %.3f ± %.3f s-1,kd2 = %.3f ± %.3f s-1, kd3 = %.3f ± %.3f s-1 \n\r",beta1(1),CSE(1),beta1(2),CSE(2),beta1(3),CSE(3))                           ;
            %fprintf('kaprime1  = %.3f ± %.3f s^-1, kaprime2  = %.3f ± %.3f s^-1 \n\r',beta1(4),CSE(4),beta1(5),CSE(5))                                                  ;
            fprintf('kaprime1  = %.3f ± %.3f s^-1, kaprime2  = %.3f ± %.3f s^-1, kaprime3  = %.3f ± %.3f s^-1,\n\r',beta1(4),CSE(4),beta1(5),CSE(5),beta1(6),CSE(6))     ;
            fprintf('kex1 = %.3f ± %.3f s^-1, kex2 = %.3f ± %.3f s^-1\n\r',beta1(7),CSE(7),beta1(8),CSE(8))                                                              ;
            fprintf('k12 = %.3f  s^-1, k21 = %.3f s^-1\n\r',k12,k21)                                                                 ;
            fprintf('R1 = %.3f ± %.3f s^-1, R2 = %.3f ± %.3f s^-1, R3 = %.3f ± %.3f s^-1, Rf = %.3f ± %.3f s^-1\n\r',beta1(9),CSE(9),beta1(10),CSE(10),beta1(11),CSE(11),beta1(12),CSE(12))   ;
            
     case 'IrHH->H2 M 5*5'
                        
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            MH1a = data.Ha(1);       MH1b = data.Hb(1);   MH2 = data.H2(1);     MH2a = data.Ha2(1);        MH2b = data.Hb2(1)        ; %
            k12  = 0.264;  k21 = 0.565 ; RF = 1/1.885                                                                                ; % 0.284   0.54  0.994       0.565   0.57  0.522
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1,1,1],5*Nt,1)                                                    ;
            ydata                         = [data.Ha; data.Hb; data.Ha2; data.Hb2; data.H2]                                          ; % the three sets of ydata values 
            I                             = [0.3, 0.5, 0.01, 0.5,-0.2,1,0.8, 0.8,1.385]                                                 ; % Initial parameters   kd1,  kd2, kaprime1, kaprime2, kex1, kex2,   R1,   R2,   Rf,
            % Initial guess                                                         (MH1a, MH1b, MH2a, MH2b, MH2, k12,  k21,  kd1,  kd2, kaprime1, kaprime2, kex1, kex2,   R1,   R2,   Rf, x)                                                                                                     
            fitfunc                       = @(I, x)SABRE_hydrogen_exchange_Lmatrix_5(MH1a, MH1b, MH2a, MH2b, MH2, k12,  k21, I(1),  0.4,    I(3),     I(4), I(5), I(6), I(7), I(8), RF, x)            ; 
            y0                            = SABRE_hydrogen_exchange_Lmatrix_5(MH1a, MH1b, MH2a, MH2b,        MH2, k12,  k21, I(1), I(2),    I(3),     I(4), I(5), I(6), I(7), I(8), RF, xdata_fit )   ;
            semilogx(data.dmix, data.Ha, 'oblack', data.dmix, data.Hb, 'or',data.dmix, data.Ha2, 'og',data.dmix, data.Hb2,'Xg',data.dmix, data.H2,'Ob')                                                    ; %,data.dmix,reshape(y0,Nt,6)
            hold on
            semilogx(data.dmix, y0(1:29), '--black', data.dmix, y0(30:58), '--r',data.dmix, y0(59:87), '--g',data.dmix, y0(88:116),'green',data.dmix, y0(117:145),'blue')                                  ;
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, I)                                                      ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("Results of Model IrHH->H2 M 5*5 has: \n")                                                                       ;
            fprintf("kd1 = %.3f ± %.3f s-1,kd2 = %.3f ± %.3f s-1 \n\r",beta1(1),CSE(1),beta1(2),CSE(2))                              ;
            fprintf('kaprime1  = %.3f ± %.3f s^-1, kaprime2  = %.3f ± %.3f s^-1 \n\r',beta1(3),CSE(3),beta1(4),CSE(4))               ;
            fprintf('kex1 = %.3f ± %.3f s^-1, kex2 = %.3f ± %.3f s^-1\n\r',beta1(5),CSE(5),beta1(6),CSE(6))                          ;
            fprintf('k12 = %.3f  s^-1, k21 = %.3f s^-1\n\r',k12,k21)                                                                 ;
            fprintf('R1 = %.3f ± %.3f s^-1, R2 = %.3f ± %.3f s^-1, Rf = %.3f ± %.3f s^-1\n\r',beta1(7),CSE(7),beta1(8),CSE(8),RF, 0)   ; % beta1(9),CSE(9)
            
     case 'IrHH->H2 M 4*4'
                        
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            MH1a = data.Ha(1);       MH1b = data.Hb(1);   MH2 = data.H2(1);     MH2a = data.Ha2(1)                                   ; %
            k12  = 0.384                                                                                                             ; %
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1,1],4*Nt,1)                                                      ;
            ydata                         = [data.Ha; data.Hb; data.Ha2; data.H2]                                                    ; % the three sets of ydata values 
            I                             = [0.6, 0.3, 0.2, 0.5, -0.2, 1, 0.7, 0.565,1]                                              ; % Initial parameters   kd1,  kaprime1,     kex1,     kex2,  R1,   R2,    Rf,
            % Initial guess                                                                (MH1a, MH1b, MH2a, MH2,  k12,  kd1,  kd2, kaprime1, kaprime2, kex1,    kex2,    R1,   R2,    Rf,     x)
            fitfunc                       = @(I, x)SABRE_hydrogen_exchange_Lmatrix_4(MH1a, MH1b, MH2a, MH2,  k12, I(1), I(2),     I(3),     I(4), I(5),    I(6),  I(7), I(8),  I(9),     x)           ; 
            y0                            = SABRE_hydrogen_exchange_Lmatrix_4(MH1a, MH1b, MH2a, MH2,                k12, I(1), I(2),     I(3),     I(4), I(5),    I(6),  I(7), I(8),  I(9), xdata_fit )      ;
            semilogx(data.dmix, data.Ha, 'oblack', data.dmix, data.Hb, 'or',data.dmix, data.Ha2, 'og',data.dmix, data.H2,'Ob')               ; %,data.dmix,reshape(y0,Nt,6)
            hold on
            semilogx(data.dmix, y0(1:29), '--black', data.dmix, y0(30:58), '--r',data.dmix, y0(59:87), '--g',data.dmix, y0(88:116),'blue')   ;
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, I)                                                      ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("Results of Model IrHH->H2 M 4*4 has: \n")                                                                       ;
            fprintf("kd1 = %.3f ± %.3f s-1,kaprime1 = %.3f ± %.3f s-1 \n\r",beta1(1),CSE(1),beta1(3),CSE(3))                         ;
            fprintf("kd2 = %.3f ± %.3f s-1,kaprime2 = %.3f ± %.3f s-1 \n\r",beta1(2),CSE(2),beta1(4),CSE(4))                         ;
            fprintf('kex1  = %.3f ± %.3f s^-1, kex2  = %.3f ± %.3f s^-1 \n\r',beta1(5),CSE(5),beta1(6),CSE(6))                       ;
            fprintf('R1 = %.3f ± %.3f s^-1, R2 = %.3f ± %.3f s^-1, Rf = %.3f ± %.3f s^-1\n\r',beta1(7),CSE(7),beta1(8),CSE(8),beta1(9),CSE(9))   ;
            
      case '([1]<->[2])->H2 pre 3*3'
                        
            fprintf("start to fit with IrHH->H2 Matrix model \n")
            data.C1  = data.Ha + data.Hb;               data.C2  = data.Ha2 + data.Hb2;
            MHC1 = data.C1(1);       MHC2 = data.C2(1);   MH2 = data.H2(1);                                                            %
            data.r                        = zeros(size(data.dmix))                                                                   ; %     
            Nt                            = length(data.dmix)                                                                        ;
            xdata_fit                     = reshape(data.dmix*[1,1,1],3*Nt,1)                                                        ;
            ydata                         = [data.C1; data.C2; data.H2]                                                              ; % the three sets of ydata values 
            I                             = [0.4, 0.3, 0.2, 0.5, 0.02, 0.02, 0.8, 1]                                                 ; % Initial parameters    k12,  k21,  kd1,  kd2, kaprime1, kaprime2,    R1,    Rf,
            % Initial guess                                                                   (MHC1, MHC2, MH2,  k12,  k21,  kd1,  kd2, kaprime1, kaprime2,    R1,    Rf, x)
            fitfunc                       = @(I, x)SABRE_hydrogen_exchange_Lmatrix_pre3(MHC1, MHC2, MH2, I(1), I(2), I(3), I(4),     I(5),     I(6),  I(7),  I(8), x)               ; 
            y0                            = SABRE_hydrogen_exchange_Lmatrix_pre3(MHC1, MHC2, MH2,        I(1), I(2), I(3), I(4),     I(5),     I(6),  I(7),  I(8), xdata_fit )      ;
            semilogx(data.dmix, data.C1, 'oblack', data.dmix, data.C2, 'or',data.dmix, data.H2,'Ob')                                 ; %,data.dmix,reshape(y0,Nt,6)
            hold on
            semilogx(data.dmix, y0(1:29), 'black', data.dmix, y0(30:58), 'r',data.dmix, y0(59:87), 'blue')                           ;
            title("initial guess")
            [beta1,R,J,CovB]              = nlinfit(xdata_fit,ydata,fitfunc, I)                                                      ;
            coefficients_standard_errors  = sqrt(diag(CovB))                                                                         ;
            CSE                           = coefficients_standard_errors                                                             ; 
            fprintf("Results of Model ([1]<->[2])->H2 pre 3*3 has: \n")                                                              ;
            fprintf("kd1 = %.3f ± %.3f s-1,kaprime1 = %.3f ± %.3f s-1 \n\r",beta1(3),CSE(3),beta1(5),CSE(5))                         ;
            fprintf("kd2 = %.3f ± %.3f s-1,kaprime2 = %.3f ± %.3f s-1 \n\r",beta1(4),CSE(4),beta1(6),CSE(6))                         ;
            fprintf('k1->2  = %.3f ± %.3f    k2->1  = %.3f ± %.3f s^-1\n\r',beta1(1),CSE(1),beta1(2),CSE(2))                         ;
            fprintf('R1 = R2 = %.3f ± %.3f s^-1, Rf = %.3f ± %.3f s^-1\n\r',beta1(7),CSE(7),beta1(8),CSE(8))                         ;
                 
end
%%
          switch Model_type
         
       case {'IrHH->H2','IrHH->H2_normal','IrHH->H2 with 3R-alpha','IrHH->H2 with 3R'}
           
            figure("Color","white",'DefaultAxesFontSize',20,'Position',[1500 500 850 800])                                          ;
            semilogx(data.dmix0, data.Ha, 'oblack', data.dmix0, data.Hb, 'or',data.dmix0, data.H2, 'ob')                            ;
            hold on                                                                                                                 ;
            x_fit                     = logspace(log10(min(data.dmix0)),log10(max(data.dmix0)),100)'                                ;
            x_fit_fit                 = reshape(x_fit*[1,1,1],3*length(x_fit),1)                                                    ;
            y_fit                     = fitfunc(beta1,x_fit_fit)                                                                    ;
            semilogx(x_fit, y_fit(1:100), 'black',x_fit, y_fit(101:200), 'red',x_fit, y_fit(201:end), 'blue')                       ;
            ylim([-0.1 1.1])                                                                                                        ;
            xlim([data.dmix(1) data.dmix(end)])
            legend('Ha', 'Hb', 'H2','Fitting')                                                                                      ;
            %PAPER                     = [ x_fit, y_fit(1:100), y_fit(201:end),y_fit(101:200)]                                       ;
            %PAPER                    = [x_fit, y_fit(1:100), y_fit(101:200),y_fit(201:end)]                                        ;
            %csvwrite(File+'_simdata_'+ Model_type +'.csv',PAPER) 
            
       case '([1]<->[2])->H2 pre 3*3'
           
            figure("Color","white",'DefaultAxesFontSize',20,'Position',[1500 500 850 800])                                          ;
            semilogx(data.dmix0, data.C1, 'oblack', data.dmix0, data.C2, 'or',data.dmix0, data.H2, 'ob')                            ;
            hold on                                                                                                                 ;
            x_fit                     = logspace(log10(min(data.dmix0)),log10(max(data.dmix0)),100)'                                ;
            x_fit_fit                 = reshape(x_fit*[1,1,1],3*length(x_fit),1)                                                    ;
            y_fit                     = fitfunc(beta1,x_fit_fit)                                                                    ;
            semilogx(x_fit, y_fit(1:100), 'black',x_fit, y_fit(101:200), 'red',x_fit, y_fit(201:end), 'blue')                       ;
            ylim([-0.1 1.1])                                                                                                        ;
            xlim([data.dmix(1) data.dmix(end)])
            legend('Ha', 'Hb', 'H2','Fitting')  
            
       case {'IrHH->H2 M 6*6','IrHH->H2 M 6*6 fmincon'}
           
            figure("Color","white",'DefaultAxesFontSize',20,'Position',[1500 500 850 800])                                          ;
            semilogx(data.dmix0, data.Ha, 'oblack', data.dmix0, data.Hb, 'or',data.dmix0, data.Ha2, 'og',data.dmix0, data.Hb2,'Xg',data.dmix0, data.H3,'Xr',data.dmix0, data.H2,'Ob')  ;
            hold on                                                                                                                 ;
            x_fit                     = logspace(log10(min(data.dmix0)),log10(max(data.dmix0)),100)'                                ;
            x_fit_fit                 = reshape(x_fit*[1,1,1,1,1,1],6*length(x_fit),1)                                              ;
            y_fit                     = fitfunc(beta1,x_fit_fit)                                                                    ;
            semilogx(x_fit, y_fit(1:100), '--black',x_fit, y_fit(101:200), '--red',x_fit, y_fit(201:300), '--green',x_fit, y_fit(301:400), 'green',x_fit, y_fit(401:500), 'red',x_fit, y_fit(501:end), 'blue')  ;
            ylim([-0.2 1.1])                                                                                                        ;
            xlim([data.dmix(1) data.dmix(end)])
            legend('Ha', 'Hb', 'H2','Ha2', 'Hb2', 'H3','H2','Fitting')                                                                   ;
            PAPER                     = [ x_fit, y_fit(1:100), y_fit(101:200),y_fit(201:300),y_fit(301:400),y_fit(401:500),y_fit(501:end)]  ;
            %PAPER                    = [x_fit, y_fit(1:100), y_fit(101:200),y_fit(201:end)]                                        ;
            %csvwrite(File+'_simdata_'+ Model_type +'.csv',PAPER) 
            
       case {'IrHH->H2 M 5*5'}
           
            figure("Color","white",'DefaultAxesFontSize',20,'Position',[1500 500 850 800])                                          ;
            semilogx(data.dmix0, data.Ha, 'oblack', data.dmix0, data.Hb, 'or',data.dmix0, data.Ha2, 'og',data.dmix0, data.Hb2,'Xg',data.dmix0, data.H2,'Ob')  ;
            hold on                                                                                                                 ;
            x_fit                     = logspace(log10(min(data.dmix0)),log10(max(data.dmix0)),100)'                                ;
            x_fit_fit                 = reshape(x_fit*[1,1,1,1,1],5*length(x_fit),1)                                              ;
            y_fit                     = fitfunc(beta1,x_fit_fit)                                                                    ;
            semilogx(x_fit, y_fit(1:100), '--black',x_fit, y_fit(101:200), '--red',x_fit, y_fit(201:300), '--green',x_fit, y_fit(301:400), 'green',x_fit, y_fit(401:500), 'blue')  ;
            ylim([-0.2 1.1])                                                                                                        ;
            xlim([data.dmix(1) data.dmix(end)])
            legend('Ha', 'Hb','Ha2', 'Hb2', 'H2','Fitting')                                                                   ;
           % PAPER                     = [ x_fit, y_fit(1:100), y_fit(101:200),y_fit(201:300),y_fit(301:400),y_fit(401:end)]  ;
            %csvwrite(File+'_simdata_'+ Model_type +'.csv',PAPER) 
       case {'IrHH->H2 M 4*4'}
           
            figure("Color","white",'DefaultAxesFontSize',20,'Position',[1500 500 850 800])                                          ;
            semilogx(data.dmix0, data.Ha, 'oblack', data.dmix0, data.Hb, 'or',data.dmix0, data.Ha2, 'og',data.dmix0, data.H2,'Ob')  ;
            hold on                                                                                                                 ;
            x_fit                     = logspace(log10(min(data.dmix0)),log10(max(data.dmix0)),100)'                                ;
            x_fit_fit                 = reshape(x_fit*[1,1,1,1],4*length(x_fit),1)                                                  ;
            y_fit                     = fitfunc(beta1,x_fit_fit)                                                                    ;
            semilogx(x_fit, y_fit(1:100), '--black',x_fit, y_fit(101:200), '--red',x_fit, y_fit(201:300), '--green',x_fit, y_fit(301:end), 'blue')  ;
            ylim([-0.2 1.1])                                                                                                        ;
            xlim([data.dmix(1) data.dmix(end)])
            legend('Ha', 'Hb', 'H2','Ha2', 'H2','Fitting')                                                                   ;
           % PAPER                     = [ x_fit, y_fit(1:100), y_fit(101:200),y_fit(201:300),y_fit(301:400),y_fit(401:end)]  ;
           % csvwrite(File+'_simdata_'+ Model_type +'.csv',PAPER) 
          end 
toc