%%% this script is made to analyse 2 set of data of carbon exchange from [1] to [2] and [2] to [1] using global fit. 
% 
clear all
close all 
tic
p = LoadPath('CA')                                                         ;
              

%%
%%%%%%%%%%%%%%%%%%%%%% Load all data  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Folder         = "IrHH_data\"                                              ;
 
Files{1}       = "kd_pyr_pressure_[1]\kd_pyr_press_5PSI_[1]_100mM_new.txt"       ; %  || kd_pyr_pressure_[1]\kd_pyr_press_75PSI_[1]_100mM   kd_IrHH_DMSO_[3]\kd_IrHH_Ha_DMSO_150mM_[3]_norm
Files{2}       = "kd_pyr_pressure_[2]\kd_pyr_press_5PSI_[2]_100mM_new.txt"       ; % kd_IrHH_DMSO_[3]\kd_IrHH_Hb_DMSO_150mM_[3]_norm.txt

[x    ,y    , L]  = combinealldata(Folder,Files)                           ;   

%%%%% Initial parameter guesses %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I               = [0.13,     0.3,    0.02, ...
                  0.02,     0.1,     0.3, ...
                     1,       1,        ]                                  ;
                                 
%% %%%%%%%%%%%%%% Fitting all combined data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MC1  = y(cumsum([1,3*L(1)]))                                               ;   
MC2  = y(cumsum([L(1)+1,2*L(1)+L(2)]))                                     ;  
MCF  = y(cumsum([2*L(1)+1,L(1)+2*L(2)]))                                   ;
        

RF   = 1/30.27                                                             ;
% MC1  = [1,0]                                                              ;
%%%%%%%%%%%%%% Define the fitting function %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fitting_function = @(I, x) Fit_CA_2_SEPP_SABRE_kd_pyr( MC1,     MC2,   MCF,  RF, L, ...  % add fnorm
                                                      I(1),    I(2),  I(3),   ...
                                                      I(4),    I(5),  I(6),   ...
                                                      I(7),    I(8),   x   ); % (MC1, MC2, MCF, Rf, L, kd1, kd2, kaprime1, kaprime2, kex1, kex2, R1, R2,  x)
                                                          
%%%%%%%%%%%%%% Perform the fitting using nlinfit %%%%%%%%%%%%%%%%%%%%%%%%%%
[beta1,R,J,CovB] = nlinfit(x, y, fitting_function, I)                      ; % ydata_norm

ysim             = fitting_function(beta1,x)                               ;
SE               = sqrt(diag(CovB))                                        ;

%%%%%%%%%%%%%%%%%%% Display the fitted parameters %%%%%%%%%%%%%%%%%%%%%%%%%
kd1                = beta1(1)                                              ;
kd2                = beta1(2)                                              ;
kaprime1           = beta1(3)                                              ;
kaprime2           = beta1(4)                                              ;
kex1               = beta1(5)                                              ;
kex2               = beta1(6)                                              ;
R1                 = beta1(7)                                              ;
R2                 = beta1(8)                                              ;


disp('Fitted Parameters:')                                                 ;
fprintf('kd1  = %.3f ± %.3f s^-1 ',[kd1,SE(1)])                            ;
fprintf('kd2  = %.3f ± %.3f s^-1 \n\r',[kd2,SE(2)])                        ;
fprintf('kex1 = %.3f ± %.3f s^-1 ',[kex1,SE(5)])                           ;
fprintf('kex2 = %.3f ± %.3f s^-1 \n\r',[kex2,SE(6)])                       ;
fprintf('kaprime1 = %.3f ± %.3f s^-1 ',[kaprime1,SE(3)])                   ;
fprintf('kaprime2 = %.3f ± %.3f s^-1 \n\r',[kaprime2,SE(4)])               ;
fprintf('R1 = %.3f ± %.3f s^-1 ',[R1,SE(7)])                               ;
fprintf('R2 = %.3f ± %.3f s^-1 ',[R2,SE(8)])                               ;


%% 

x_fit               = logspace(log10(0.5*min(x)),log10(max(x)*2),100)'     ;
x_fit2              = reshape(x_fit*[1,1,1],3*length(x_fit),1)             ;
L2                  = length(x_fit)*[1,1,1]                                ;    
                                         

fitting_function2   = @(I, x) Fit_CA_2_SEPP_SABRE_kd_pyr ( MC1, MC2,   MCF,      RF,     L2,   ...
                                                                         I(1),    I(2),  I(3),   ...
                                                                         I(4),    I(5),  I(6),   ...
                                                                         I(7),    I(8),   x_fit2)      ; % (MHa1,MHb1,MH21,MHa2,MHb2,MH22,kd,kex,kaprime,RHa,RHb,RH2,L, x)  

y_fit               = fitting_function2(beta1,x_fit2)                      ; 

%%%%%%%%%%%%%%%%%% Plotting %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plotNkinetics_loc(2,x,y,x_fit,y_fit,L,L2)                                  ; 

%%%%%%%%%%%% Increasing the number of the fitting points %%%%%%%%%%%%%%%%%%
                                                    

toc
%%
%%%%%%%%%%%%%%%%%%Functions%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [xdata, ydata, data_lengths] = combinealldata(Folder, Files)
                            num_files = length(Files)                      ;

                            xdata     = []                                 ;
                            ydata     = []                                 ;
                         data_lengths = zeros(1, num_files)                ;

    for i = 1: num_files
        loaddata        = load(fullfile(Folder, Files{i}))                 ;
        xdata           = [xdata, loaddata(:, 1)']                         ;                
        ydata           = [ydata, loaddata(:, 2)',loaddata(:, 3)',loaddata(:, 4)'] ;
        data_lengths(i) = length(loaddata(:, 1))                           ;
    end

    xdata = xdata'                                                         ;
    ydata = ydata'                                                         ;
end

function RMSD  = plotNkinetics_loc(N,x,y,xsim,ysim,L,L_fit)
                               
    figure("Color","white",'DefaultAxesFontSize',20,'Position',[700 0 850 800])
    start          = 1                                                     ; 
    startx         = 1                                                     ;
    start_fit      = 1                                                     ; 
    startx_fit     = 1                                                     ;
    colors         = [0.4940 0.1840 0.5560 ;0.9290 0.6940 0.1250 ; 1 0 0]  ; 
    markers        = {'O', 'O', 'O'}                                       ; % Define markers
    c = 0                                                                  ;
    for k = 1:N
        subplot(N,1,k)
        h = semilogx(x(startx:startx-1+L(k))',reshape(y(start:start+3*L(k)-1),L(k),3),'MarkerSize',13,'LineStyle', 'none', 'LineWidth',2);
        colororder(colors)
        set(h, {'Marker'}, markers(:))                                     ; % Apply markers to each line
        set(gca,'linewidth',2,'Fontsize',26)                               ;
        hold on
        semilogx(xsim,reshape(ysim(start_fit:start_fit+3*L_fit(k)-1),L_fit(k),3), "LineWidth",2) ;
         
        start      = start  + 3*L(k)                                       ;        
        startx     = startx + L(k)                                         ;
        start_fit  = start_fit + 3*L_fit(k)                                ;
        
        hold off
        % legend(["[3]^a","[3]^b","H_2"],'Fontsize',32)                      
        % xlabel("𝜏_e (s)",'Fontsize',22)
        % ylabel("Integrals (norm.)")

        xticks([  0.01 0.1 1 10 100])
        xticklabels({  '0.01', '0.1', '1', '10' , '100'})
        yticks([ 0 0.2 0.4 0.6 0.8 1])
        yticklabels({'0', '0.2', '0.4', '0.6', '0.8', '1'})
        axis([0.004,200,-0.2,1.1])        
        % legend('boxoff')
        c = c + L(k)                                                       ;
    end
 %  saveas(gcf,'G:\.shortcut-targets-by-id\1sm8HgUXJdPnYH1dZK3HxYOaHt5pDZtFT\IrHH_exchange_paper_2024\Figure\Figure_S14_pyr_DMSO_[1]_[2]\DMSO_300mM.png')
end