% [1] Chiavazza et al 10.1016/j.jmr.2012.11.016
% [2] Kharkov et al 10.1039/d1cp05537b
%% set the system
J=830; % Hz [1] page 3, results 14.5+-0.1 Hz
S=1; % spin of X nuclei of fast relaxation
tauJ = 1*1e-7; % s [1] page 3, results 1.0+-0.1 ms
tauD = 77.3*1e-12; % s [2] page 3, 77ps
tauCSA = 5*77.3*1e-12; % s [2] page 3, 77ps
CSA = 130e-6;
rIS = 2.3*1e-10; % m
B = logspace(log10(0.3e-1), log10(10),100)'; % T
gamma1H=26.75222080*1e7; %rad/(s*T)
gamma13C=6.728286*1e7;  %rad/(s*T)
gamma14N=1.93377980*1e7; %rad/(s*T)
gamma15N=-2.7126189*1e7; %rad/(s*T)
T10 = 40; % T1 at high fields (s)
R10 = 1/T10;


%% data
D=[32	0.495211307	2.01934
48	0.921361773	1.08535
64	1.337154028	0.747857
125	3.166140032	0.315842
250	7.810791389	0.128028
500	11.79690449	0.084768
1000	13.10226184	0.0763227
2000	13.23928148	0.0755328
4000	12.53233342	0.0797936
9470	7.247164547	0.137985];

%%
R1_J = R_J(J,S,gamma15N,gamma1H,tauJ,B) ;
R1_DD = R_DD(rIS,S,gamma15N,gamma1H,tauD,B);
R1_CSA = R_CSA(gamma15N*B,CSA,tauCSA);
R1 = R10 + R1_J + R1_DD + R1_CSA;

figure("Color","White")
loglog(B,[R1_J,R1_DD,R1_CSA,R1],"LineWidth",3);
xlabel("Magnetic field, T")
ylabel("R_1 (s^{-1})")
hold on
semilogx(D(:,1)/1000,D(:,3),"o","MarkerSize",10,"LineWidth",2)
hold off
title("15N NAA T1 relaxation: R_1")
legend(["R_{1J}","R_{1DD}","R_{1CSA}","R_{1,all}","Exp data"])
axis([0.02,20,0,2.3])

figure("Color","White")
loglog(B,[1./R1_J,1./R1_DD,1./R1_CSA,1./R1],"LineWidth",3);
xlabel("Magnetic field, T")
ylabel("T_1 (s^{-1})")
hold on
semilogx(D(:,1)/1000,D(:,2),"o","MarkerSize",10,"LineWidth",2)
hold off
title("15N NAA T1 relaxation: T_1")
legend(["T_{1J}","T_{1DD}","T_{1CSA}","T_{1,all}","Exp data"])
axis([0.02,20,0,30])

function R=R_J(J,S,gammaI,gammaS,tau,B)
    R=8*pi^2*J^2/3*(S*(S+1))*tau*Jw0((gammaI-gammaS)*B*tau);
end

function R=R_CSA(w0,CSA,tau)
    % [2], eq 4 ESI
    R=(1/5)*(w0*CSA).^2*tau.*Jw0(w0*tau);
end

function R=R_DD(rIS,S,gammaI,gammaS,tau,B)
    % eq 2 DOI: 10.1039/d1cp05537b
    mu0 = 1.25663706127*1e-6; % N*A-2
    hbar = 1.054571817*1e-34; % J⋅s
    R=2/3*(mu0*hbar*gammaI*gammaS/(4*pi*rIS^3))^2*S*(S+1)*tau*(Jw0((gammaI-gammaS)*B*tau)+3*Jw0(gammaI*B*tau)+6*Jw0((gammaI+gammaS)*B*tau));
end

function J=Jw0(x)
    J=1./(1+(x).^2);
end

